@@ -95,7 +95,7 @@ QTransform QuickItemNodeInstance::transform() const
 QObject *QuickItemNodeInstance::parent() const
 {
     if (!quickItem() || !quickItem()->parentItem())
-        return 0;
+        return nullptr;
 
     return quickItem()->parentItem();
 }
@@ -156,7 +156,8 @@ void QuickItemNodeInstance::createEffectItem(bool createEffectItem)
     s_createEffectItem = createEffectItem;
 }
 
-void QuickItemNodeInstance::initialize(const ObjectNodeInstance::Pointer &objectNodeInstance)
+void QuickItemNodeInstance::initialize(const ObjectNodeInstance::Pointer &objectNodeInstance,
+                                       InstanceContainer::NodeFlags flags)
 {
 
     if (instanceId() == 0) {
@@ -167,10 +168,11 @@ void QuickItemNodeInstance::initialize(const ObjectNodeInstance::Pointer &object
 
     if (quickItem()->window()) {
         if (s_createEffectItem || instanceId() == 0)
-            designerSupport()->refFromEffectItem(quickItem());
+            designerSupport()->refFromEffectItem(quickItem(),
+                                                 !flags.testFlag(InstanceContainer::ParentTakesOverRendering));
     }
 
-    ObjectNodeInstance::initialize(objectNodeInstance);
+    ObjectNodeInstance::initialize(objectNodeInstance, flags);
     quickItem()->update();
 }
 
@@ -191,14 +193,14 @@ void QuickItemNodeInstance::doComponentComplete()
 {
     ObjectNodeInstance::doComponentComplete();
 
+    QmlPrivateGate::disableTextCursor(quickItem());
+
+    QmlPrivateGate::emitComponentComplete(quickItem());
+
     QQmlProperty contentItemProperty(quickItem(), "contentItem", engine());
     if (contentItemProperty.isValid())
         m_contentItem = contentItemProperty.read().value<QQuickItem*>();
 
-    QmlPrivateGate::disableTextCursor(quickItem());
-
-    DesignerSupport::emitComponentCompleteSignalForAttachedProperty(quickItem());
-
     quickItem()->update();
 }
 
@@ -382,10 +384,12 @@ QImage QuickItemNodeInstance::renderPreviewImage(const QSize &previewImageSize)
     QRectF previewItemBoundingRect = boundingRect();
 
     if (previewItemBoundingRect.isValid() && quickItem()) {
+        static double devicePixelRatio = qgetenv("FORMEDITOR_DEVICE_PIXEL_RATIO").toDouble();
+        const QSize size = previewImageSize * devicePixelRatio;
         if (quickItem()->isVisible()) {
-            return designerSupport()->renderImageForItem(quickItem(), previewItemBoundingRect, previewImageSize);
+            return designerSupport()->renderImageForItem(quickItem(), previewItemBoundingRect, size);
         } else {
-            QImage transparentImage(previewImageSize, QImage::Format_ARGB32_Premultiplied);
+            QImage transparentImage(size, QImage::Format_ARGB32_Premultiplied);
             transparentImage.fill(Qt::transparent);
             return transparentImage;
         }
@@ -769,7 +773,7 @@ QPair<PropertyName, ServerNodeInstance> QuickItemNodeInstance::anchor(const Prop
 
     while (targetObject) {
         if (nodeInstanceServer()->hasInstanceForObject(targetObject))
-            return qMakePair(targetName, nodeInstanceServer()->instanceForObject(targetObject));
+            return {targetName, nodeInstanceServer()->instanceForObject(targetObject)};
         else
             targetObject = parentObject(targetObject);
     }
@@ -795,8 +799,8 @@ bool QuickItemNodeInstance::isAnchoredBySibling() const
 
 QQuickItem *QuickItemNodeInstance::quickItem() const
 {
-    if (object() == 0)
-        return 0;
+    if (object() == nullptr)
+        return nullptr;
 
     return static_cast<QQuickItem*>(object());
 }
