@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -66,19 +61,19 @@ namespace {
 class LastVisibleSymbolAt: protected SymbolVisitor
 {
     Symbol *root;
-    unsigned line;
-    unsigned column;
+    int line;
+    int column;
     Symbol *symbol;
 
 public:
     LastVisibleSymbolAt(Symbol *root)
-        : root(root), line(0), column(0), symbol(0) {}
+        : root(root), line(0), column(0), symbol(nullptr) {}
 
-    Symbol *operator()(unsigned line, unsigned column)
+    Symbol *operator()(int line, int column)
     {
         this->line = line;
         this->column = column;
-        this->symbol = 0;
+        this->symbol = nullptr;
         accept(root);
         if (! symbol)
             symbol = root;
@@ -102,14 +97,14 @@ protected:
 class FindScopeAt: protected SymbolVisitor
 {
     TranslationUnit *_unit;
-    unsigned _line;
-    unsigned _column;
+    int _line;
+    int _column;
     Scope *_scope;
 
 public:
     /** line and column should be 1-based */
-    FindScopeAt(TranslationUnit *unit, unsigned line, unsigned column)
-        : _unit(unit), _line(line), _column(column), _scope(0) {}
+    FindScopeAt(TranslationUnit *unit, int line, int column)
+        : _unit(unit), _line(line), _column(column), _scope(nullptr) {}
 
     Scope *operator()(Symbol *symbol)
     {
@@ -123,18 +118,18 @@ protected:
         if (! _scope) {
             Scope *scope = symbol;
 
-            for (unsigned i = 0; i < scope->memberCount(); ++i) {
+            for (int i = 0; i < scope->memberCount(); ++i) {
                 accept(scope->memberAt(i));
 
                 if (_scope)
                     return false;
             }
 
-            unsigned startLine, startColumn;
+            int startLine, startColumn;
             _unit->getPosition(scope->startOffset(), &startLine, &startColumn);
 
             if (_line > startLine || (_line == startLine && _column >= startColumn)) {
-                unsigned endLine, endColumn;
+                int endLine, endColumn;
                 _unit->getPosition(scope->endOffset(), &endLine, &endColumn);
 
                 if (_line < endLine || (_line == endLine && _column < endColumn))
@@ -147,56 +142,56 @@ protected:
 
     using SymbolVisitor::visit;
 
-    virtual bool preVisit(Symbol *)
+    bool preVisit(Symbol *) override
     { return ! _scope; }
 
-    virtual bool visit(UsingNamespaceDirective *) { return false; }
-    virtual bool visit(UsingDeclaration *) { return false; }
-    virtual bool visit(NamespaceAlias *) { return false; }
-    virtual bool visit(Declaration *) { return false; }
-    virtual bool visit(Argument *) { return false; }
-    virtual bool visit(TypenameArgument *) { return false; }
-    virtual bool visit(BaseClass *) { return false; }
-    virtual bool visit(ForwardClassDeclaration *) { return false; }
+    bool visit(UsingNamespaceDirective *) override { return false; }
+    bool visit(UsingDeclaration *) override { return false; }
+    bool visit(NamespaceAlias *) override { return false; }
+    bool visit(Declaration *) override { return false; }
+    bool visit(Argument *) override { return false; }
+    bool visit(TypenameArgument *) override { return false; }
+    bool visit(BaseClass *) override { return false; }
+    bool visit(ForwardClassDeclaration *) override { return false; }
 
-    virtual bool visit(Enum *symbol)
+    bool visit(Enum *symbol) override
     { return process(symbol); }
 
-    virtual bool visit(Function *symbol)
+    bool visit(Function *symbol) override
     { return process(symbol); }
 
-    virtual bool visit(Namespace *symbol)
+    bool visit(Namespace *symbol) override
     { return process(symbol); }
 
-    virtual bool visit(Class *symbol)
+    bool visit(Class *symbol) override
     { return process(symbol); }
 
-    virtual bool visit(Block *symbol)
+    bool visit(Block *symbol) override
     { return process(symbol); }
 
-    virtual bool visit(Template *symbol)
+    bool visit(Template *symbol) override
     {
         if (Symbol *decl = symbol->declaration()) {
-            if (decl->isFunction() || decl->isClass())
+            if (decl->isFunction() || decl->isClass() || decl->isDeclaration())
                 return process(symbol);
         }
         return true;
     }
 
     // Objective-C
-    virtual bool visit(ObjCBaseClass *) { return false; }
-    virtual bool visit(ObjCBaseProtocol *) { return false; }
-    virtual bool visit(ObjCForwardClassDeclaration *) { return false; }
-    virtual bool visit(ObjCForwardProtocolDeclaration *) { return false; }
-    virtual bool visit(ObjCPropertyDeclaration *) { return false; }
+    bool visit(ObjCBaseClass *) override { return false; }
+    bool visit(ObjCBaseProtocol *) override { return false; }
+    bool visit(ObjCForwardClassDeclaration *) override { return false; }
+    bool visit(ObjCForwardProtocolDeclaration *) override { return false; }
+    bool visit(ObjCPropertyDeclaration *) override { return false; }
 
-    virtual bool visit(ObjCClass *symbol)
+    bool visit(ObjCClass *symbol) override
     { return process(symbol); }
 
-    virtual bool visit(ObjCProtocol *symbol)
+    bool visit(ObjCProtocol *symbol) override
     { return process(symbol); }
 
-    virtual bool visit(ObjCMethod *symbol)
+    bool visit(ObjCMethod *symbol) override
     { return process(symbol); }
 };
 
@@ -214,10 +209,10 @@ public:
           errorCount(0)
     { }
 
-    virtual void report(int level,
-                        const StringLiteral *fileId,
-                        unsigned line, unsigned column,
-                        const char *format, va_list ap)
+    void report(int level,
+                const StringLiteral *fileId,
+                int line, int column,
+                const char *format, va_list ap) override
     {
         if (level == Error) {
             ++errorCount;
@@ -233,8 +228,7 @@ public:
         if (fileName != doc->fileName())
             return;
 
-        QString message;
-        message.vsprintf(format, ap);
+        const QString message = QString::vasprintf(format, ap);
 
 #ifndef DO_NOT_DUMP_ALL_PARSER_ERRORS
         {
@@ -271,7 +265,7 @@ private:
 
 Document::Document(const QString &fileName)
     : _fileName(QDir::cleanPath(fileName)),
-      _globalNamespace(0),
+      _globalNamespace(nullptr),
       _revision(0),
       _editorRevision(0),
       _checkMode(0)
@@ -285,16 +279,17 @@ Document::Document(const QString &fileName)
                                                                       localFileName.size());
     _translationUnit = new TranslationUnit(_control, fileId);
     _translationUnit->setLanguageFeatures(LanguageFeatures::defaultFeatures());
-    (void) _control->switchTranslationUnit(_translationUnit);
 }
 
 Document::~Document()
 {
     delete _translationUnit;
-    _translationUnit = 0;
-    delete _control->diagnosticClient();
-    delete _control;
-    _control = 0;
+    _translationUnit = nullptr;
+    if (_control) {
+        delete _control->diagnosticClient();
+        delete _control;
+    }
+    _control = nullptr;
 }
 
 Control *Document::control() const
@@ -302,6 +297,25 @@ Control *Document::control() const
     return _control;
 }
 
+Control *Document::swapControl(Control *newControl)
+{
+    if (newControl) {
+        const StringLiteral *fileId = newControl->stringLiteral(_translationUnit->fileId()->chars(),
+                                                                _translationUnit->fileId()->size());
+        const auto newTranslationUnit = new TranslationUnit(newControl, fileId);
+        newTranslationUnit->setLanguageFeatures(_translationUnit->languageFeatures());
+        delete _translationUnit;
+        _translationUnit = newTranslationUnit;
+    } else {
+        delete _translationUnit;
+        _translationUnit = nullptr;
+    }
+
+    Control *oldControl = _control;
+    _control = newControl;
+    return oldControl;
+}
+
 unsigned Document::revision() const
 {
     return _revision;
@@ -361,9 +375,9 @@ void Document::appendMacro(const Macro &macro)
 }
 
 void Document::addMacroUse(const Macro &macro,
-                           unsigned bytesOffset, unsigned bytesLength,
-                           unsigned utf16charsOffset, unsigned utf16charLength,
-                           unsigned beginLine,
+                           int bytesOffset, int bytesLength,
+                           int utf16charsOffset, int utf16charLength,
+                           int beginLine,
                            const QVector<MacroArgumentReference> &actuals)
 {
     MacroUse use(macro,
@@ -383,7 +397,7 @@ void Document::addMacroUse(const Macro &macro,
 }
 
 void Document::addUndefinedMacroUse(const QByteArray &name,
-                                    unsigned bytesOffset, unsigned utf16charsOffset)
+                                    int bytesOffset, int utf16charsOffset)
 {
     QByteArray copy(name.data(), name.size());
     UndefinedMacroUse use(copy, bytesOffset, utf16charsOffset);
@@ -454,7 +468,7 @@ void Document::setSkipFunctionBody(bool skipFunctionBody)
     _translationUnit->setSkipFunctionBody(skipFunctionBody);
 }
 
-unsigned Document::globalSymbolCount() const
+int Document::globalSymbolCount() const
 {
     if (! _globalNamespace)
         return 0;
@@ -462,7 +476,7 @@ unsigned Document::globalSymbolCount() const
     return _globalNamespace->memberCount();
 }
 
-Symbol *Document::globalSymbolAt(unsigned index) const
+Symbol *Document::globalSymbolAt(int index) const
 {
     return _globalNamespace->memberAt(index);
 }
@@ -513,23 +527,17 @@ QString Document::functionAt(int line, int column, int *lineOpeningDeclaratorPar
         return QString();
 
     // We found the function scope
-    if (lineOpeningDeclaratorParenthesis) {
-        unsigned line;
-        translationUnit()->getPosition(scope->startOffset(), &line);
-        *lineOpeningDeclaratorParenthesis = static_cast<int>(line);
-    }
+    if (lineOpeningDeclaratorParenthesis)
+        translationUnit()->getPosition(scope->startOffset(), lineOpeningDeclaratorParenthesis);
 
-    if (lineClosingBrace) {
-        unsigned line;
-        translationUnit()->getPosition(scope->endOffset(), &line);
-        *lineClosingBrace = static_cast<int>(line);
-    }
+    if (lineClosingBrace)
+        translationUnit()->getPosition(scope->endOffset(), lineClosingBrace);
 
     const QList<const Name *> fullyQualifiedName = LookupContext::fullyQualifiedName(scope);
     return Overview().prettyName(fullyQualifiedName);
 }
 
-Scope *Document::scopeAt(unsigned line, unsigned column)
+Scope *Document::scopeAt(int line, int column)
 {
     FindScopeAt findScopeAt(_translationUnit, line, column);
     if (Scope *scope = findScopeAt(_globalNamespace))
@@ -537,22 +545,22 @@ Scope *Document::scopeAt(unsigned line, unsigned column)
     return globalNamespace();
 }
 
-Symbol *Document::lastVisibleSymbolAt(unsigned line, unsigned column) const
+Symbol *Document::lastVisibleSymbolAt(int line, int column) const
 {
     LastVisibleSymbolAt lastVisibleSymbolAt(globalNamespace());
     return lastVisibleSymbolAt(line, column);
 }
 
-const Macro *Document::findMacroDefinitionAt(unsigned line) const
+const Macro *Document::findMacroDefinitionAt(int line) const
 {
     foreach (const Macro &macro, _definedMacros) {
         if (macro.line() == line)
             return &macro;
     }
-    return 0;
+    return nullptr;
 }
 
-const Document::MacroUse *Document::findMacroUseAt(unsigned utf16charsOffset) const
+const Document::MacroUse *Document::findMacroUseAt(int utf16charsOffset) const
 {
     foreach (const Document::MacroUse &use, _macroUses) {
         if (use.containsUtf16charOffset(utf16charsOffset)
@@ -560,10 +568,10 @@ const Document::MacroUse *Document::findMacroUseAt(unsigned utf16charsOffset) co
             return &use;
         }
     }
-    return 0;
+    return nullptr;
 }
 
-const Document::UndefinedMacroUse *Document::findUndefinedMacroUseAt(unsigned utf16charsOffset) const
+const Document::UndefinedMacroUse *Document::findUndefinedMacroUseAt(int utf16charsOffset) const
 {
     foreach (const Document::UndefinedMacroUse &use, _undefinedMacroUses) {
         if (use.containsUtf16charOffset(utf16charsOffset)
@@ -571,7 +579,7 @@ const Document::UndefinedMacroUse *Document::findUndefinedMacroUseAt(unsigned ut
                     + QString::fromUtf8(use.name(), use.name().size()).length()))
             return &use;
     }
-    return 0;
+    return nullptr;
 }
 
 Document::Ptr Document::create(const QString &fileName)
@@ -602,17 +610,17 @@ void Document::setLanguageFeatures(LanguageFeatures features)
         tu->setLanguageFeatures(features);
 }
 
-void Document::startSkippingBlocks(unsigned utf16charsOffset)
+void Document::startSkippingBlocks(int utf16charsOffset)
 {
     _skippedBlocks.append(Block(0, 0, utf16charsOffset, 0));
 }
 
-void Document::stopSkippingBlocks(unsigned utf16charsOffset)
+void Document::stopSkippingBlocks(int utf16charsOffset)
 {
     if (_skippedBlocks.isEmpty())
         return;
 
-    unsigned start = _skippedBlocks.back().utf16charsBegin();
+    int start = _skippedBlocks.back().utf16charsBegin();
     if (start > utf16charsOffset)
         _skippedBlocks.removeLast(); // Ignore this block, it's invalid.
     else
@@ -702,7 +710,8 @@ void Document::releaseSourceAndAST()
     if (!_keepSourceAndASTCount.deref()) {
         _source.clear();
         _translationUnit->release();
-        _control->squeeze();
+        if (_control)
+            _control->squeeze();
     }
 }
 
@@ -740,17 +749,17 @@ bool Snapshot::isEmpty() const
     return _documents.isEmpty();
 }
 
-Snapshot::const_iterator Snapshot::find(const Utils::FileName &fileName) const
+Snapshot::const_iterator Snapshot::find(const Utils::FilePath &fileName) const
 {
     return _documents.find(fileName);
 }
 
-void Snapshot::remove(const Utils::FileName &fileName)
+void Snapshot::remove(const Utils::FilePath &fileName)
 {
     _documents.remove(fileName);
 }
 
-bool Snapshot::contains(const Utils::FileName &fileName) const
+bool Snapshot::contains(const Utils::FilePath &fileName) const
 {
     return _documents.contains(fileName);
 }
@@ -758,13 +767,28 @@ bool Snapshot::contains(const Utils::FileName &fileName) const
 void Snapshot::insert(Document::Ptr doc)
 {
     if (doc) {
-        _documents.insert(Utils::FileName::fromString(doc->fileName()), doc);
+        _documents.insert(Utils::FilePath::fromString(doc->fileName()), doc);
         m_deps.files.clear(); // Will trigger re-build when accessed.
     }
 }
 
+static QList<Macro> macrosDefinedUntilLine(const QList<Macro> &macros, int line)
+{
+    QList<Macro> filtered;
+
+    foreach (const Macro &macro, macros) {
+        if (macro.line() <= line)
+            filtered.append(macro);
+        else
+            break;
+    }
+
+    return filtered;
+}
+
 Document::Ptr Snapshot::preprocessedDocument(const QByteArray &source,
-                                             const Utils::FileName &fileName) const
+                                             const Utils::FilePath &fileName,
+                                             int withDefinedMacrosFromDocumentUntilLine) const
 {
     Document::Ptr newDoc = Document::create(fileName.toString());
     if (Document::Ptr thisDocument = document(fileName)) {
@@ -774,10 +798,15 @@ Document::Ptr Snapshot::preprocessedDocument(const QByteArray &source,
         newDoc->_resolvedIncludes = thisDocument->_resolvedIncludes;
         newDoc->_unresolvedIncludes = thisDocument->_unresolvedIncludes;
         newDoc->setLanguageFeatures(thisDocument->languageFeatures());
+        if (withDefinedMacrosFromDocumentUntilLine != -1) {
+            newDoc->_definedMacros = macrosDefinedUntilLine(thisDocument->_definedMacros,
+                                                            withDefinedMacrosFromDocumentUntilLine);
+        }
     }
 
     FastPreprocessor pp(*this);
-    const QByteArray preprocessedCode = pp.run(newDoc, source);
+    const bool mergeDefinedMacrosOfDocument = !newDoc->_definedMacros.isEmpty();
+    const QByteArray preprocessedCode = pp.run(newDoc, source, mergeDefinedMacrosOfDocument);
     newDoc->setUtf8Source(preprocessedCode);
     return newDoc;
 }
@@ -822,7 +851,7 @@ QList<Snapshot::IncludeLocation> Snapshot::includeLocationsOfDocument(const QStr
     return result;
 }
 
-Utils::FileNameList Snapshot::filesDependingOn(const Utils::FileName &fileName) const
+Utils::FilePathList Snapshot::filesDependingOn(const Utils::FilePath &fileName) const
 {
     updateDependencyTable();
     return m_deps.filesDependingOn(fileName);
@@ -851,7 +880,7 @@ void Snapshot::allIncludesForDocument_helper(const QString &fileName, QSet<QStri
     }
 }
 
-Document::Ptr Snapshot::document(const Utils::FileName &fileName) const
+Document::Ptr Snapshot::document(const Utils::FilePath &fileName) const
 {
     return _documents.value(fileName);
 }
