@@ -91,7 +91,7 @@ CMakeBuildSettingsWidget::CMakeBuildSettingsWidget(CMakeBuildConfiguration *bc)
     setDisplayName(tr("CMake"));
 
     auto vbox = new QVBoxLayout(this);
-    vbox->setMargin(0);
+    vbox->setContentsMargins(0, 0, 0, 0);
     auto container = new Utils::DetailsWidget;
     container->setState(Utils::DetailsWidget::NoSummary);
     vbox->addWidget(container);
@@ -100,7 +100,7 @@ CMakeBuildSettingsWidget::CMakeBuildSettingsWidget(CMakeBuildConfiguration *bc)
     container->setWidget(details);
 
     auto mainLayout = new QGridLayout(details);
-    mainLayout->setMargin(0);
+    mainLayout->setContentsMargins(0, 0, 0, 0);
     mainLayout->setColumnStretch(1, 10);
 
     auto project = static_cast<CMakeProject *>(bc->project());
@@ -252,7 +252,7 @@ CMakeBuildSettingsWidget::CMakeBuildSettingsWidget(CMakeBuildConfiguration *bc)
         m_showProgressTimer.start();
     });
 
-    if (m_buildConfiguration->isParsing())
+    if (project->isParsing())
         m_showProgressTimer.start();
     else {
         m_configModel->setConfiguration(m_buildConfiguration->configurationFromCMake());
@@ -319,6 +319,7 @@ CMakeBuildSettingsWidget::CMakeBuildSettingsWidget(CMakeBuildConfiguration *bc)
         });
         QModelIndex idx = m_configModel->indexForItem(item);
         idx = m_configTextFilterModel->mapFromSource(m_configFilterModel->mapFromSource(idx));
+        m_configView->setFocus();
         m_configView->scrollTo(idx);
         m_configView->setCurrentIndex(idx);
         m_configView->edit(idx);
@@ -363,7 +364,7 @@ void CMakeBuildSettingsWidget::setWarning(const QString &message)
 
 void CMakeBuildSettingsWidget::updateButtonState()
 {
-    const bool isParsing = m_buildConfiguration->isParsing();
+    const bool isParsing = m_buildConfiguration->project()->isParsing();
     const bool hasChanges = m_configModel->hasChanges();
     m_resetButton->setEnabled(hasChanges && !isParsing);
     m_reconfigureButton->setEnabled((hasChanges || m_configModel->hasCMakeChanges()) && !isParsing);
@@ -408,7 +409,7 @@ void CMakeBuildSettingsWidget::setConfigurationForCMake()
 
 void CMakeBuildSettingsWidget::updateSelection(const QModelIndex &current, const QModelIndex &previous)
 {
-    Q_UNUSED(previous);
+    Q_UNUSED(previous)
 
     m_editButton->setEnabled(current.isValid() && current.flags().testFlag(Qt::ItemIsEditable));
     m_unsetButton->setEnabled(current.isValid() && current.flags().testFlag(Qt::ItemIsSelectable));
