@@ -49,7 +49,7 @@ class CORE_EXPORT IFileWizardExtension : public QObject
     Q_OBJECT
 public:
     IFileWizardExtension();
-    ~IFileWizardExtension();
+    ~IFileWizardExtension() override;
     static QList<IFileWizardExtension *> allFileWizardExtensions();
 
     /* Return a list of pages to be added to the Wizard (empty list if not
