@@ -103,7 +103,7 @@ public:
      *
      * This method needs to be thread safe!
      */
-    virtual bool isVcsFileOrDirectory(const Utils::FileName &fileName) const = 0;
+    virtual bool isVcsFileOrDirectory(const Utils::FilePath &fileName) const = 0;
 
     /*!
      * Returns whether files in this directory should be managed with this
@@ -213,10 +213,23 @@ public:
      * \a extraArgs are passed on to the command being run.
      */
     virtual ShellCommand *createInitialCheckoutCommand(const QString &url,
-                                                       const Utils::FileName &baseDirectory,
+                                                       const Utils::FilePath &baseDirectory,
                                                        const QString &localName,
                                                        const QStringList &extraArgs);
 
+    class CORE_EXPORT RepoUrl {
+    public:
+        RepoUrl(const QString &location);
+
+        QString protocol;
+        QString userName;
+        QString host;
+        QString path;
+        quint16 port = 0;
+        bool isValid = false;
+    };
+    virtual RepoUrl getRepoUrl(const QString &location) const;
+
 signals:
     void repositoryChanged(const QString &repository);
     void filesChanged(const QStringList &files);
@@ -226,10 +239,10 @@ private:
     TopicCache *m_topicCache;
 };
 
-Q_DECLARE_OPERATORS_FOR_FLAGS(Core::IVersionControl::SettingsFlags)
-
 } // namespace Core
 
+Q_DECLARE_OPERATORS_FOR_FLAGS(Core::IVersionControl::SettingsFlags)
+
 #if defined(WITH_TESTS)
 
 #include <QSet>
@@ -245,8 +258,8 @@ public:
     { }
     ~TestVersionControl() override;
 
-    bool isVcsFileOrDirectory(const Utils::FileName &fileName) const final
-    { Q_UNUSED(fileName); return false; }
+    bool isVcsFileOrDirectory(const Utils::FilePath &fileName) const final
+    { Q_UNUSED(fileName) return false; }
 
     void setManagedDirectories(const QHash<QString, QString> &dirs);
     void setManagedFiles(const QSet<QString> &files);
