@@ -335,23 +335,27 @@ int ProjectTreeWidget::expandedCount(Node *node)
 
 void ProjectTreeWidget::rowsInserted(const QModelIndex &parent, int start, int end)
 {
+    if (m_delayedRename.isEmpty())
+        return;
     Node *node = m_model->nodeForIndex(parent);
     QTC_ASSERT(node, return);
-    int i = start;
-    while (i <= end) {
+    for (int i = start; i <= end && !m_delayedRename.isEmpty(); ++i) {
         QModelIndex idx = m_model->index(i, 0, parent);
         Node *n = m_model->nodeForIndex(idx);
-        if (n && n->filePath() == m_delayedRename) {
+        if (!n)
+            continue;
+        const int renameIdx = m_delayedRename.indexOf(n->filePath());
+        if (renameIdx != -1) {
             m_view->setCurrentIndex(idx);
-            m_delayedRename.clear();
-            break;
+            m_delayedRename.removeAt(renameIdx);
         }
-        ++i;
     }
 }
 
 Node *ProjectTreeWidget::nodeForFile(const FilePath &fileName)
 {
+    if (fileName.isEmpty())
+        return nullptr;
     Node *bestNode = nullptr;
     int bestNodeExpandCount = INT_MAX;
 
@@ -374,10 +378,7 @@ Node *ProjectTreeWidget::nodeForFile(const FilePath &fileName)
         if (ProjectNode *projectNode = project->rootProjectNode()) {
             projectNode->forEachGenericNode([&](Node *node) {
                 if (node->filePath() == fileName) {
-                    if (!bestNode) {
-                        bestNode = node;
-                        bestNodeExpandCount = ProjectTreeWidget::expandedCount(node);
-                    } else if (priority(node) < priority(bestNode)) {
+                    if (!bestNode || priority(node) < priority(bestNode)) {
                         bestNode = node;
                         bestNodeExpandCount = ProjectTreeWidget::expandedCount(node);
                     } else if (priority(node) == priority(bestNode)) {
@@ -460,14 +461,14 @@ void ProjectTreeWidget::editCurrentItem()
 void ProjectTreeWidget::renamed(const FilePath &oldPath, const FilePath &newPath)
 {
     update();
-    Q_UNUSED(oldPath);
+    Q_UNUSED(oldPath)
     if (!currentNode() || currentNode()->filePath() != newPath) {
         // try to find the node
         Node *node = nodeForFile(newPath);
         if (node)
             m_view->setCurrentIndex(m_model->indexForNode(node));
         else
-            m_delayedRename = newPath;
+            m_delayedRename << newPath;
     }
 }
 
@@ -503,7 +504,7 @@ void ProjectTreeWidget::setCurrentItem(Node *node)
 
 void ProjectTreeWidget::handleCurrentItemChange(const QModelIndex &current)
 {
-    Q_UNUSED(current);
+    Q_UNUSED(current)
     ProjectTree::nodeChanged(this);
 }
 
