@@ -84,6 +84,8 @@ public:
     Qt::ItemFlags flags(int column) const override;
     bool setData(int column, const QVariant &, int role) override;
 
+    ProjectPanelFactory *factory() const { return m_factory; }
+
 protected:
     ProjectPanelFactory *m_factory = nullptr;
     QPointer<Project> m_project;
@@ -213,8 +215,6 @@ public:
     {
         switch (role) {
         case Qt::DisplayRole:
-            return m_project->displayName();
-
         case ProjectDisplayNameRole:
             return m_project->displayName();
 
@@ -283,6 +283,13 @@ public:
         return activeItem ? activeItem->index() : QModelIndex();
     }
 
+    TreeItem *itemForProjectPanel(Core::Id panelId)
+    {
+        return m_miscItem->findChildAtLevel(1, [panelId](const TreeItem *item){
+            return static_cast<const MiscSettingsPanelItem *>(item)->factory()->id() == panelId;
+        });
+    }
+
 private:
     int m_currentChildIndex = 0; // Start with Build & Run.
     Project *m_project = nullptr;
@@ -376,7 +383,7 @@ public:
 
         m_projectSelection = new QComboBox;
         m_projectSelection->setModel(&m_comboBoxModel);
-        connect(m_projectSelection, static_cast<void(QComboBox::*)(int)>(&QComboBox::activated),
+        connect(m_projectSelection, QOverload<int>::of(&QComboBox::activated),
                 this, &ProjectWindowPrivate::projectSelected, Qt::QueuedConnection);
 
         SessionManager *sessionManager = SessionManager::instance();
@@ -499,6 +506,14 @@ public:
             item->setData(0, QVariant(), ItemActivatedDirectlyRole);
     }
 
+    void activateProjectPanel(Core::Id panelId)
+    {
+        if (ProjectItem *projectItem = m_projectsModel.rootItem()->childAt(0)) {
+            if (TreeItem *item = projectItem->itemForProjectPanel(panelId))
+                itemActivated(item->index());
+        }
+    }
+
     void openContextMenu(const QPoint &pos)
     {
         QMenu menu;
@@ -546,18 +561,14 @@ public:
         QString importDir = QFileDialog::getExistingDirectory(ICore::mainWindow(),
                                                               ProjectWindow::tr("Import Directory"),
                                                               dir);
-        FileName path = FileName::fromString(importDir);
+        FilePath path = FilePath::fromString(importDir);
 
         Target *lastTarget = nullptr;
         BuildConfiguration *lastBc = nullptr;
         for (const BuildInfo &info : projectImporter->import(path, false)) {
             Target *target = project->target(info.kitId);
-            if (!target) {
-                std::unique_ptr<Target> newTarget = project->createTarget(KitManager::kit(info.kitId));
-                target = newTarget.get();
-                if (newTarget)
-                    project->addTarget(std::move(newTarget));
-            }
+            if (!target)
+                target = project->addTargetForKit(KitManager::kit(info.kitId));
             if (target) {
                 projectImporter->makePersistent(target->kit());
                 BuildConfiguration *bc = info.factory()->create(target, info);
@@ -612,6 +623,11 @@ ProjectWindow::ProjectWindow()
     setContextMenuPolicy(Qt::CustomContextMenu);
 }
 
+void ProjectWindow::activateProjectPanel(Core::Id panelId)
+{
+    d->activateProjectPanel(panelId);
+}
+
 ProjectWindow::~ProjectWindow() = default;
 
 QSize SelectorDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
