@@ -29,16 +29,18 @@
 #include "propertyeditortransaction.h"
 #include <qmldesignerconstants.h>
 #include <qmldesignerplugin.h>
+#include <qmltimeline.h>
 
 #include <qmlobjectnode.h>
 #include <nodemetainfo.h>
 #include <variantproperty.h>
 #include <bindingproperty.h>
 
-#include <theming.h>
+#include <theme.h>
 
 #include <coreplugin/icore.h>
 #include <qmljs/qmljssimplereader.h>
+#include <utils/qtcassert.h>
 #include <utils/algorithm.h>
 #include <utils/fileutils.h>
 
@@ -46,6 +48,9 @@
 #include <QDir>
 #include <QFileInfo>
 
+#include <QLoggingCategory>
+
+static Q_LOGGING_CATEGORY(propertyEditorBenchmark, "qtc.propertyeditor.load", QtWarningMsg)
 
 static QmlJS::SimpleReaderNode::Ptr s_templateConfiguration = QmlJS::SimpleReaderNode::Ptr();
 
@@ -82,7 +87,7 @@ static QObject *variantToQObject(const QVariant &value)
     if (value.userType() == QMetaType::QObjectStar || value.userType() > QMetaType::User)
         return *(QObject **)value.constData();
 
-    return 0;
+    return nullptr;
 }
 
 namespace QmlDesigner {
@@ -91,32 +96,27 @@ PropertyEditorQmlBackend::PropertyEditorQmlBackend(PropertyEditorView *propertyE
         m_view(new Quick2PropertyEditorView), m_propertyEditorTransaction(new PropertyEditorTransaction(propertyEditor)), m_dummyPropertyEditorValue(new PropertyEditorValue()),
         m_contextObject(new PropertyEditorContextObject())
 {
-    Q_ASSERT(QFileInfo(QLatin1String(":/images/button_normal.png")).exists());
-
     m_view->engine()->setOutputWarningsToStandardError(QmlDesignerPlugin::instance()
         ->settings().value(DesignerSettingsKey::SHOW_PROPERTYEDITOR_WARNINGS).toBool());
 
-    m_view->engine()->addImportPath(propertyEditorResourcesPath());
+    m_view->engine()->addImportPath(propertyEditorResourcesPath() + "/imports");
     m_dummyPropertyEditorValue->setValue(QLatin1String("#000000"));
     context()->setContextProperty(QLatin1String("dummyBackendValue"), m_dummyPropertyEditorValue.data());
     m_contextObject->setBackendValues(&m_backendValuesPropertyMap);
+    m_contextObject->setModel(propertyEditor->model());
     m_contextObject->insertInQmlContext(context());
 
-    Theming::insertTheme(&m_themeProperties);
-    context()->setContextProperty(QLatin1String("creatorTheme"), &m_themeProperties);
-
-    QObject::connect(&m_backendValuesPropertyMap, &DesignerPropertyMap::valueChanged, propertyEditor, &PropertyEditorView::changeValue);
+    QObject::connect(&m_backendValuesPropertyMap, &DesignerPropertyMap::valueChanged,
+                     propertyEditor, &PropertyEditorView::changeValue);
 }
 
-PropertyEditorQmlBackend::~PropertyEditorQmlBackend()
-{
-}
+PropertyEditorQmlBackend::~PropertyEditorQmlBackend() = default;
 
 void PropertyEditorQmlBackend::setupPropertyEditorValue(const PropertyName &name, PropertyEditorView *propertyEditor, const QString &type)
 {
     QmlDesigner::PropertyName propertyName(name);
     propertyName.replace('.', '_');
-    PropertyEditorValue *valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(backendValuesPropertyMap().value(QString::fromUtf8(propertyName))));
+    auto valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(backendValuesPropertyMap().value(QString::fromUtf8(propertyName))));
     if (!valueObject) {
         valueObject = new PropertyEditorValue(&backendValuesPropertyMap());
         QObject::connect(valueObject, &PropertyEditorValue::valueChanged, &backendValuesPropertyMap(), &DesignerPropertyMap::valueChanged);
@@ -131,19 +131,13 @@ void PropertyEditorQmlBackend::setupPropertyEditorValue(const PropertyName &name
 
 }
 
-static PropertyNameList layoutAttachedPropertiesNames()
-{
-    PropertyNameList propertyNames;
-    propertyNames << "alignment" << "column" << "columnSpan" << "fillHeight" << "fillWidth"
-                  << "maximumHeight" << "maximumWidth" << "minimumHeight" << "minimumWidth"
-                  << "preferredHeight" << "preferredWidth" << "row"<< "rowSpan";
-
-    return propertyNames;
-}
-
 QVariant properDefaultLayoutAttachedProperties(const QmlObjectNode &qmlObjectNode, const PropertyName &propertyName)
 {
-    QVariant value = qmlObjectNode.modelValue(propertyName);
+    const QVariant value = qmlObjectNode.modelValue("Layout." + propertyName);
+    QVariant marginsValue = qmlObjectNode.modelValue("Layout.margins");
+
+    if (!marginsValue.isValid())
+        marginsValue.setValue(0.0);
 
     if (value.isValid())
         return value;
@@ -163,6 +157,11 @@ QVariant properDefaultLayoutAttachedProperties(const QmlObjectNode &qmlObjectNod
      if ("columnSpan" == propertyName || "rowSpan" == propertyName)
          return 1;
 
+     if ("topMargin" == propertyName || "bottomMargin" == propertyName ||
+         "leftMargin" == propertyName || "rightMargin" == propertyName ||
+         "margins" == propertyName)
+         return marginsValue;
+
     return QVariant();
 }
 
@@ -170,9 +169,12 @@ void PropertyEditorQmlBackend::setupLayoutAttachedProperties(const QmlObjectNode
 {
     if (QmlItemNode(qmlObjectNode).isInLayout()) {
 
-        static const PropertyNameList propertyNames = layoutAttachedPropertiesNames();
+        static const PropertyNameList propertyNames =
+            {"alignment", "column", "columnSpan", "fillHeight", "fillWidth", "maximumHeight", "maximumWidth",
+                "minimumHeight", "minimumWidth", "preferredHeight", "preferredWidth", "row", "rowSpan",
+                "topMargin", "bottomMargin", "leftMargin", "rightMargin", "margins"};
 
-        foreach (const PropertyName &propertyName, propertyNames) {
+        for (const PropertyName &propertyName : propertyNames) {
             createPropertyEditorValue(qmlObjectNode, "Layout." + propertyName, properDefaultLayoutAttachedProperties(qmlObjectNode, propertyName), propertyEditor);
         }
     }
@@ -185,11 +187,13 @@ void PropertyEditorQmlBackend::createPropertyEditorValue(const QmlObjectNode &qm
 {
     PropertyName propertyName(name);
     propertyName.replace('.', '_');
-    PropertyEditorValue *valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(backendValuesPropertyMap().value(QString::fromUtf8(propertyName))));
+    auto valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(backendValuesPropertyMap().value(QString::fromUtf8(propertyName))));
     if (!valueObject) {
         valueObject = new PropertyEditorValue(&backendValuesPropertyMap());
         QObject::connect(valueObject, &PropertyEditorValue::valueChanged, &backendValuesPropertyMap(), &DesignerPropertyMap::valueChanged);
         QObject::connect(valueObject, &PropertyEditorValue::expressionChanged, propertyEditor, &PropertyEditorView::changeExpression);
+        QObject::connect(valueObject, &PropertyEditorValue::exportPopertyAsAliasRequested, propertyEditor, &PropertyEditorView::exportPopertyAsAlias);
+        QObject::connect(valueObject, &PropertyEditorValue::removeAliasExportRequested, propertyEditor, &PropertyEditorView::removeAliasExport);
         backendValuesPropertyMap().insert(QString::fromUtf8(propertyName), QVariant::fromValue(valueObject));
     }
     valueObject->setName(name);
@@ -217,7 +221,7 @@ void PropertyEditorQmlBackend::setValue(const QmlObjectNode & qmlObjectNode, con
 {
     PropertyName propertyName = name;
     propertyName.replace('.', '_');
-    PropertyEditorValue *propertyValue = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value(QString::fromUtf8(propertyName))));
+    auto propertyValue = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value(QString::fromUtf8(propertyName))));
     if (propertyValue) {
         propertyValue->setValue(value);
 
@@ -263,22 +267,32 @@ PropertyEditorValue *PropertyEditorQmlBackend::propertyValueForName(const QStrin
 
 void PropertyEditorQmlBackend::setup(const QmlObjectNode &qmlObjectNode, const QString &stateName, const QUrl &qmlSpecificsFile, PropertyEditorView *propertyEditor)
 {
-    if (!qmlObjectNode.isValid())
-        return;
-
     if (qmlObjectNode.isValid()) {
+
+        m_contextObject->setModel(propertyEditor->model());
+
+        qCInfo(propertyEditorBenchmark) << Q_FUNC_INFO;
+
+        QElapsedTimer time;
+        if (propertyEditorBenchmark().isInfoEnabled())
+            time.start();
+
         foreach (const PropertyName &propertyName, qmlObjectNode.modelNode().metaInfo().propertyNames())
             createPropertyEditorValue(qmlObjectNode, propertyName, qmlObjectNode.instanceValue(propertyName), propertyEditor);
 
         setupLayoutAttachedProperties(qmlObjectNode, propertyEditor);
 
+        // model node
+        m_backendModelNode.setup(qmlObjectNode.modelNode());
+        context()->setContextProperty(QLatin1String("modelNodeBackend"), &m_backendModelNode);
+
         // className
-        PropertyEditorValue *valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value(QLatin1String("className"))));
+        auto valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value(QLatin1String("className"))));
         if (!valueObject)
             valueObject = new PropertyEditorValue(&m_backendValuesPropertyMap);
         valueObject->setName("className");
         valueObject->setModelNode(qmlObjectNode.modelNode());
-        valueObject->setValue(qmlObjectNode.modelNode().simplifiedTypeName());
+        valueObject->setValue(m_backendModelNode.simplifiedTypeName());
         QObject::connect(valueObject, &PropertyEditorValue::valueChanged, &backendValuesPropertyMap(), &DesignerPropertyMap::valueChanged);
         m_backendValuesPropertyMap.insert(QLatin1String("className"), QVariant::fromValue(valueObject));
 
@@ -287,7 +301,7 @@ void PropertyEditorQmlBackend::setup(const QmlObjectNode &qmlObjectNode, const Q
         if (!valueObject)
             valueObject = new PropertyEditorValue(&m_backendValuesPropertyMap);
         valueObject->setName("id");
-        valueObject->setValue(qmlObjectNode.id());
+        valueObject->setValue(m_backendModelNode.nodeId());
         QObject::connect(valueObject, &PropertyEditorValue::valueChanged, &backendValuesPropertyMap(), &DesignerPropertyMap::valueChanged);
         m_backendValuesPropertyMap.insert(QLatin1String("id"), QVariant::fromValue(valueObject));
 
@@ -299,18 +313,25 @@ void PropertyEditorQmlBackend::setup(const QmlObjectNode &qmlObjectNode, const Q
 
         context()->setContextProperty(QLatin1String("transaction"), m_propertyEditorTransaction.data());
 
-        // model node
-        m_backendModelNode.setup(qmlObjectNode.modelNode());
-        context()->setContextProperty(QLatin1String("modelNodeBackend"), &m_backendModelNode);
+        qCInfo(propertyEditorBenchmark) << "anchors:" << time.elapsed();
+
+        qCInfo(propertyEditorBenchmark) << "context:" << time.elapsed();
 
         contextObject()->setSpecificsUrl(qmlSpecificsFile);
 
+        qCInfo(propertyEditorBenchmark) << "specifics:" << time.elapsed();
+
         contextObject()->setStateName(stateName);
         if (!qmlObjectNode.isValid())
             return;
         context()->setContextProperty(QLatin1String("propertyCount"), QVariant(qmlObjectNode.modelNode().properties().count()));
 
         contextObject()->setIsBaseState(qmlObjectNode.isInBaseState());
+
+        contextObject()->setHasAliasExport(qmlObjectNode.isAliasExported());
+
+        contextObject()->setHasActiveTimeline(QmlTimeline::hasActiveTimeline(qmlObjectNode.view()));
+
         contextObject()->setSelectionChanged(false);
 
         contextObject()->setSelectionChanged(false);
@@ -323,10 +344,14 @@ void PropertyEditorQmlBackend::setup(const QmlObjectNode &qmlObjectNode, const Q
         } else {
             contextObject()->setMajorVersion(-1);
             contextObject()->setMinorVersion(-1);
-             contextObject()->setMajorQtQuickVersion(-1);
+            contextObject()->setMajorQtQuickVersion(-1);
+            contextObject()->setMinorQtQuickVersion(-1);
         }
 
         contextObject()->setMajorQtQuickVersion(qmlObjectNode.view()->majorQtQuickVersion());
+        contextObject()->setMinorQtQuickVersion(qmlObjectNode.view()->minorQtQuickVersion());
+
+        qCInfo(propertyEditorBenchmark) << "final:" << time.elapsed();
     } else {
         qWarning() << "PropertyEditor: invalid node for setup";
     }
@@ -339,7 +364,7 @@ void PropertyEditorQmlBackend::initialSetup(const TypeName &typeName, const QUrl
     foreach (const PropertyName &propertyName, metaInfo.propertyNames())
         setupPropertyEditorValue(propertyName, propertyEditor, QString::fromUtf8(metaInfo.propertyTypeName(propertyName)));
 
-    PropertyEditorValue *valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value(QLatin1String("className"))));
+    auto valueObject = qobject_cast<PropertyEditorValue*>(variantToQObject(m_backendValuesPropertyMap.value(QLatin1String("className"))));
     if (!valueObject)
         valueObject = new PropertyEditorValue(&m_backendValuesPropertyMap);
     valueObject->setName("className");
@@ -376,25 +401,84 @@ QString PropertyEditorQmlBackend::propertyEditorResourcesPath() {
     return Core::ICore::resourcePath() + QStringLiteral("/qmldesigner/propertyEditorQmlSources");
 }
 
-QString PropertyEditorQmlBackend::templateGeneration(NodeMetaInfo type,
-                                                     NodeMetaInfo superType,
-                                                     const QmlObjectNode &objectNode)
+inline bool dotPropertyHeuristic(const QmlObjectNode &node, const NodeMetaInfo &type, const PropertyName &name)
+{
+    if (!name.contains("."))
+        return true;
+
+    if (name.count('.') > 1)
+        return false;
+
+    QList<QByteArray> list =name.split('.');
+    const PropertyName parentProperty = list.first();
+    const PropertyName itemProperty = list.last();
+
+    TypeName typeName = type.propertyTypeName(parentProperty);
+
+    NodeMetaInfo itemInfo = node.view()->model()->metaInfo("QtQuick.Item");
+    NodeMetaInfo textInfo = node.view()->model()->metaInfo("QtQuick.Text");
+    NodeMetaInfo rectangleInfo = node.view()->model()->metaInfo("QtQuick.Rectangle");
+
+    if (itemInfo.hasProperty(itemProperty))
+        return false;
+
+    if (typeName == "font")
+        return false;
+
+    if (textInfo.isSubclassOf(typeName))
+            return false;
+
+    if (rectangleInfo.isSubclassOf(typeName))
+            return false;
+
+    return true;
+}
+
+QString PropertyEditorQmlBackend::templateGeneration(const NodeMetaInfo &type,
+                                                     const NodeMetaInfo &superType,
+                                                     const QmlObjectNode &node)
 {
     if (!templateConfiguration() || !templateConfiguration()->isValid())
         return QString();
 
+    const auto nodes = templateConfiguration()->children();
+
+    QStringList sectorTypes;
+
+    for (const QmlJS::SimpleReaderNode::Ptr &node : nodes) {
+        if (node->propertyNames().contains("separateSection"))
+            sectorTypes.append(variantToStringList(node->property("typeNames")));
+    }
+
     QStringList imports = variantToStringList(templateConfiguration()->property(QStringLiteral("imports")));
 
     QString qmlTemplate = imports.join(QLatin1Char('\n')) + QLatin1Char('\n');
-    qmlTemplate += QStringLiteral("Section {\n");
-    qmlTemplate += QStringLiteral("caption: \"%1\"\n").arg(objectNode.modelNode().simplifiedTypeName());
-    qmlTemplate += QStringLiteral("SectionLayout {\n");
+
+    qmlTemplate += "Column {\n";
+    qmlTemplate += "anchors.left: parent.left\n";
+    qmlTemplate += "anchors.right: parent.right\n";
 
     QList<PropertyName> orderedList = type.propertyNames();
-    Utils::sort(orderedList);
+    Utils::sort(orderedList, [type, &sectorTypes](const PropertyName &left, const PropertyName &right){
+        const QString typeNameLeft = QString::fromLatin1(type.propertyTypeName(left));
+        const QString typeNameRight = QString::fromLatin1(type.propertyTypeName(right));
+        if (typeNameLeft == typeNameRight)
+            return left > right;
+
+        if (sectorTypes.contains(typeNameLeft)) {
+            if (sectorTypes.contains(typeNameRight))
+                return left > right;
+            return true;
+        } else if (sectorTypes.contains(typeNameRight)) {
+            return false;
+        }
+        return left > right;
+    });
 
     bool emptyTemplate = true;
 
+    bool sectionStarted = false;
+
     foreach (const PropertyName &name, orderedList) {
 
         if (name.startsWith("__"))
@@ -405,27 +489,46 @@ QString PropertyEditorQmlBackend::templateGeneration(NodeMetaInfo type,
 
         TypeName typeName = type.propertyTypeName(name);
         //alias resolution only possible with instance
-        if (typeName == "alias" && objectNode.isValid())
-            typeName = objectNode.instanceType(name);
+        if (typeName == "alias" && node.isValid())
+            typeName = node.instanceType(name);
 
-        if (!superType.hasProperty(name) && type.propertyIsWritable(name) && !name.contains(".")) {
-            foreach (const QmlJS::SimpleReaderNode::Ptr &node, templateConfiguration()->children())
+        auto nodes = templateConfiguration()->children();
+
+        if (!superType.hasProperty(name) && type.propertyIsWritable(name) && dotPropertyHeuristic(node, type, name)) {
+
+            for (const QmlJS::SimpleReaderNode::Ptr &node : nodes) {
                 if (variantToStringList(node->property(QStringLiteral("typeNames"))).contains(QString::fromLatin1(typeName))) {
                     const QString fileName = propertyTemplatesPath() + node->property(QStringLiteral("sourceFile")).toString();
                     QFile file(fileName);
                     if (file.open(QIODevice::ReadOnly)) {
                         QString source = QString::fromUtf8(file.readAll());
                         file.close();
+                        const bool section = node->propertyNames().contains("separateSection");
+                        if (section) {
+                        } else if (!sectionStarted) {
+                            qmlTemplate += QStringLiteral("Section {\n");
+                            qmlTemplate += QStringLiteral("caption: \"%1\"\n").arg(QString::fromUtf8(type.simplifiedTypeName()));
+                            qmlTemplate += "anchors.left: parent.left\n";
+                            qmlTemplate += "anchors.right: parent.right\n";
+                            qmlTemplate += QStringLiteral("SectionLayout {\n");
+                            sectionStarted = true;
+                        }
+
                         qmlTemplate += source.arg(QString::fromUtf8(name)).arg(QString::fromUtf8(properName));
                         emptyTemplate = false;
                     } else {
                         qWarning().nospace() << "template definition source file not found:" << fileName;
                     }
                 }
+            }
         }
     }
-    qmlTemplate += QStringLiteral("}\n"); //Section
-    qmlTemplate += QStringLiteral("}\n"); //SectionLayout
+    if (sectionStarted) {
+        qmlTemplate += QStringLiteral("}\n"); //Section
+        qmlTemplate += QStringLiteral("}\n"); //SectionLayout
+    }
+
+    qmlTemplate += "}\n";
 
     if (emptyTemplate)
         return QString();
@@ -445,6 +548,36 @@ TypeName PropertyEditorQmlBackend::fixTypeNameForPanes(const TypeName &typeName)
     return fixedTypeName;
 }
 
+static NodeMetaInfo findCommonSuperClass(const NodeMetaInfo &first, const NodeMetaInfo &second)
+{
+    for (const NodeMetaInfo &info : first.superClasses()) {
+        if (second.isSubclassOf(info.typeName()))
+            return info;
+    }
+    return first;
+}
+
+NodeMetaInfo PropertyEditorQmlBackend::findCommonAncestor(const ModelNode &node)
+{
+    if (!node.isValid())
+        return {};
+
+    QTC_ASSERT(node.metaInfo().isValid(), return {});
+
+    AbstractView *view = node.view();
+
+    if (view->selectedModelNodes().count() > 1) {
+        NodeMetaInfo commonClass = node.metaInfo();
+        for (const ModelNode &currentNode :  view->selectedModelNodes()) {
+            if (currentNode.metaInfo().isValid() && !currentNode.isSubclassOf(commonClass.typeName(), -1, -1))
+              commonClass = findCommonSuperClass(currentNode.metaInfo(), commonClass);
+        }
+        return commonClass;
+    }
+
+    return node.metaInfo();
+}
+
 TypeName PropertyEditorQmlBackend::qmlFileName(const NodeMetaInfo &nodeInfo)
 {
     const TypeName fixedTypeName = fixTypeNameForPanes(nodeInfo.typeName());
@@ -481,7 +614,8 @@ QString PropertyEditorQmlBackend::fileFromUrl(const QUrl &url)
 
 bool PropertyEditorQmlBackend::checkIfUrlExists(const QUrl &url)
 {
-    return QFileInfo::exists(fileFromUrl(url));
+    const QString &file = fileFromUrl(url);
+    return !file.isEmpty() && QFileInfo::exists(file);
 }
 
 void PropertyEditorQmlBackend::emitSelectionToBeChanged()
@@ -498,17 +632,21 @@ void PropertyEditorQmlBackend::setValueforLayoutAttachedProperties(const QmlObje
 {
     PropertyName propertyName = name;
     propertyName.replace("Layout.", "");
-    setValue(qmlObjectNode,  name, properDefaultLayoutAttachedProperties(qmlObjectNode, propertyName));
+    setValue(qmlObjectNode, name, properDefaultLayoutAttachedProperties(qmlObjectNode, propertyName));
+
+    if (propertyName == "margins") {
+        const QVariant marginsValue = properDefaultLayoutAttachedProperties(qmlObjectNode, "margins");
+        setValue(qmlObjectNode, "Layout.topMargin", marginsValue);
+        setValue(qmlObjectNode, "Layout.bottomMargin", marginsValue);
+        setValue(qmlObjectNode, "Layout.leftMargin", marginsValue);
+        setValue(qmlObjectNode, "Layout.rightMargin", marginsValue);
+    }
 }
 
-QUrl PropertyEditorQmlBackend::getQmlUrlForModelNode(const ModelNode &modelNode, TypeName &className)
+QUrl PropertyEditorQmlBackend::getQmlUrlForMetaInfo(const NodeMetaInfo &metaInfo, TypeName &className)
 {
-    if (modelNode.isValid()) {
-        QList<NodeMetaInfo> hierarchy;
-        hierarchy.append(modelNode.metaInfo());
-        hierarchy.append(modelNode.metaInfo().superClasses());
-
-        foreach (const NodeMetaInfo &info, hierarchy) {
+    if (metaInfo.isValid()) {
+        foreach (const NodeMetaInfo &info, metaInfo.classHierarchy()) {
             QUrl fileUrl = fileToUrl(locateQmlFile(info, QString::fromUtf8(qmlFileName(info))));
             if (fileUrl.isValid()) {
                 className = info.typeName();
@@ -521,51 +659,47 @@ QUrl PropertyEditorQmlBackend::getQmlUrlForModelNode(const ModelNode &modelNode,
 
 QString PropertyEditorQmlBackend::locateQmlFile(const NodeMetaInfo &info, const QString &relativePath)
 {
-    QDir fileSystemDir(PropertyEditorQmlBackend::propertyEditorResourcesPath());
+    static const QDir fileSystemDir(PropertyEditorQmlBackend::propertyEditorResourcesPath());
 
-    static QDir resourcesDir(QStringLiteral(":/propertyEditorQmlSources"));
-    QDir importDir(info.importDirectoryPath() + QLatin1String(Constants::QML_DESIGNER_SUBFOLDER));
-    QDir importDirVersion(info.importDirectoryPath() + QStringLiteral(".") + QString::number(info.majorVersion()) + QLatin1String(Constants::QML_DESIGNER_SUBFOLDER));
+    const QDir resourcesDir(QStringLiteral(":/propertyEditorQmlSources"));
+    const QDir importDir(info.importDirectoryPath() + Constants::QML_DESIGNER_SUBFOLDER);
+    const QDir importDirVersion(info.importDirectoryPath() + QStringLiteral(".") + QString::number(info.majorVersion()) + Constants::QML_DESIGNER_SUBFOLDER);
 
-    const QString versionString = QStringLiteral("_") + QString::number(info.majorVersion())
-            + QStringLiteral("_")
-            + QString::number(info.minorVersion());
-
-    QString relativePathWithoutEnding = relativePath;
-    relativePathWithoutEnding.chop(4);
-    const QString relativePathWithVersion = relativePathWithoutEnding + versionString + QStringLiteral(".qml");
+    const QString relativePathWithoutEnding = relativePath.left(relativePath.count() - 4);
+    const QString relativePathWithVersion = QString("%1_%2_%3.qml").arg(relativePathWithoutEnding
+        ).arg(info.majorVersion()).arg(info.minorVersion());
 
     //Check for qml files with versions first
-    const QString withoutDirWithVersion = relativePathWithVersion.split(QStringLiteral("/")).last();
 
-    const QString withoutDir = relativePath.split(QStringLiteral("/")).last();
+    const QString withoutDir = relativePath.split(QStringLiteral("/")).constLast();
 
     if (importDirVersion.exists(withoutDir))
         return importDirVersion.absoluteFilePath(withoutDir);
 
+    const QString withoutDirWithVersion = relativePathWithVersion.split(QStringLiteral("/")).constLast();
 
+    QStringList possiblePaths = {
+        fileSystemDir.absoluteFilePath(relativePathWithVersion),
+        resourcesDir.absoluteFilePath(relativePathWithVersion),
+        fileSystemDir.absoluteFilePath(relativePath),
+        resourcesDir.absoluteFilePath(relativePath)
+    };
 
-    if (importDir.exists(relativePathWithVersion))
-        return importDir.absoluteFilePath(relativePathWithVersion);
-    if (importDir.exists(withoutDirWithVersion)) //Since we are in a subfolder of the import we do not require the directory
-        return importDir.absoluteFilePath(withoutDirWithVersion);
-    if (fileSystemDir.exists(relativePathWithVersion))
-        return fileSystemDir.absoluteFilePath(relativePathWithVersion);
-    if (resourcesDir.exists(relativePathWithVersion))
-        return resourcesDir.absoluteFilePath(relativePathWithVersion);
+    if (!importDir.isEmpty())
+        possiblePaths.append({
+            importDir.absoluteFilePath(relativePathWithVersion),
+            //Since we are in a subfolder of the import we do not require the directory
+            importDir.absoluteFilePath(withoutDirWithVersion),
+            importDir.absoluteFilePath(relativePath),
+            //Since we are in a subfolder of the import we do not require the directory
+            importDir.absoluteFilePath(withoutDir),
+        });
 
-
-    if (importDir.exists(relativePath))
-        return importDir.absoluteFilePath(relativePath);
-    if (importDir.exists(withoutDir)) //Since we are in a subfolder of the import we do not require the directory
-        return importDir.absoluteFilePath(withoutDir);
-    if (fileSystemDir.exists(relativePath))
-        return fileSystemDir.absoluteFilePath(relativePath);
-    if (resourcesDir.exists(relativePath))
-        return resourcesDir.absoluteFilePath(relativePath);
-
-    return QString();
+    return Utils::findOrDefault(possiblePaths, [](const QString &possibleFilePath) {
+        return QFileInfo::exists(possibleFilePath);
+    });
 }
 
 
 } //QmlDesigner
+
