@@ -33,6 +33,7 @@
 #include <nodeinstanceclientinterface.h>
 #include <nodeinstanceserverinterface.h>
 
+#include <QElapsedTimer>
 #include <QHash>
 #include <QImage>
 #include <QPointer>
@@ -53,7 +54,7 @@ class Project;
 
 namespace QmlDesigner {
 
-class NodeInstanceServerInterface;
+class NodeInstanceServerProxy;
 class CreateSceneCommand;
 class CreateInstancesCommand;
 class ClearSceneCommand;
@@ -63,6 +64,7 @@ class ChangeValuesCommand;
 class ChangeBindingsCommand;
 class ChangeIdsCommand;
 class RemoveInstancesCommand;
+class ChangeSelectionCommand;
 class RemovePropertiesCommand;
 class CompleteComponentCommand;
 class InformationContainer;
@@ -75,7 +77,7 @@ class QMLDESIGNERCORE_EXPORT NodeInstanceView : public AbstractView, public Node
     friend class NodeInstance;
 
 public:
-    typedef QWeakPointer<NodeInstanceView> Pointer;
+    using Pointer = QWeakPointer<NodeInstanceView>;
 
     explicit NodeInstanceView(QObject *parent = nullptr, NodeInstanceServerInterface::RunModus runModus = NodeInstanceServerInterface::NormalModus);
     ~NodeInstanceView() override;
@@ -118,6 +120,7 @@ public:
     void updatePosition(const QList<VariantProperty>& propertyList);
 
     void valuesChanged(const ValuesChangedCommand &command) override;
+    void valuesModified(const ValuesModifiedCommand &command) override;
     void pixmapChanged(const PixmapChangedCommand &command) override;
     void informationChanged(const InformationChangedCommand &command) override;
     void childrenChanged(const ChildrenChangedCommand &command) override;
@@ -133,6 +136,19 @@ public:
 
     void sendToken(const QString &token, int number, const QVector<ModelNode> &nodeVector);
 
+    void selectionChanged(const ChangeSelectionCommand &command) override;
+    void library3DItemDropped(const Drop3DLibraryItemCommand &command) override;
+    void view3DClosed(const View3DClosedCommand &command) override;
+
+    void selectedNodesChanged(const QList<ModelNode> &selectedNodeList,
+                              const QList<ModelNode> &lastSelectedNodeList) override;
+
+    void mainWindowStateChanged(Qt::WindowStates previousStates, Qt::WindowStates currentStates);
+    void mainWindowActiveChanged(bool active, bool hasPopup);
+    void enable3DView(bool enable);
+
+    void handlePuppetToCreatorCommand(const PuppetToCreatorCommand &command) override;
+
 protected:
     void timerEvent(QTimerEvent *event) override;
 
@@ -172,6 +188,7 @@ private: // functions
     ChangeBindingsCommand createChangeBindingCommand(const QList<BindingProperty> &propertyList) const;
     ChangeIdsCommand createChangeIdsCommand(const QList<NodeInstance> &instanceList) const;
     RemoveInstancesCommand createRemoveInstancesCommand(const QList<ModelNode> &nodeList) const;
+    ChangeSelectionCommand createChangeSelectionCommand(const QList<ModelNode> &nodeList) const;
     RemoveInstancesCommand createRemoveInstancesCommand(const ModelNode &node) const;
     RemovePropertiesCommand createRemovePropertiesCommand(const QList<AbstractProperty> &propertyList) const;
     RemoveSharedMemoryCommand createRemoveSharedMemoryCommand(const QString &sharedMemoryTypeName, quint32 keyNumber);
@@ -183,23 +200,27 @@ private: // functions
     void restartProcess();
     void delayedRestartProcess();
 
-private:
     void handleCrash();
+    void startPuppetTransaction();
+    void endPuppetTransaction();
+
+    // puppet to creator command handlers
+    void handlePuppetKeyPress(int key, Qt::KeyboardModifiers modifiers);
 
-private: //variables
     NodeInstance m_rootNodeInstance;
     NodeInstance m_activeStateInstance;
 
     QHash<ModelNode, NodeInstance> m_nodeInstanceHash;
     QHash<ModelNode, QImage> m_statePreviewImage;
 
-    QPointer<NodeInstanceServerInterface> m_nodeInstanceServer;
+    QPointer<NodeInstanceServerProxy> m_nodeInstanceServer;
     QImage m_baseStatePreviewImage;
-    QTime m_lastCrashTime;
+    QElapsedTimer m_lastCrashTime;
     NodeInstanceServerInterface::RunModus m_runModus;
     ProjectExplorer::Kit *m_currentKit = nullptr;
     ProjectExplorer::Project *m_currentProject = nullptr;
     int m_restartProcessTimerId;
+    RewriterTransaction m_puppetTransaction;
 };
 
 } // namespace ProxyNodeInstanceView
