@@ -46,7 +46,7 @@ static PlainTextEditorFactory *m_instance = nullptr;
 class PlainTextEditorWidget : public TextEditorWidget
 {
 public:
-    PlainTextEditorWidget() {}
+    PlainTextEditorWidget() = default;
     void finalizeInitialization() override
     {
         textDocument()->setMimeType(QLatin1String(Constants::C_TEXTEDITOR_MIMETYPE_TEXT));
@@ -65,7 +65,7 @@ PlainTextEditorFactory::PlainTextEditorFactory()
 
     setDocumentCreator([]() { return new TextDocument(Core::Constants::K_DEFAULT_TEXT_EDITOR_ID); });
     setEditorWidgetCreator([]() { return new PlainTextEditorWidget; });
-    setIndenterCreator([]() { return new NormalIndenter; });
+    setIndenterCreator([](QTextDocument *doc) { return new NormalIndenter(doc); });
     setUseGenericHighlighter(true);
 
     setEditorActionHandlers(TextEditorActionHandler::Format |
